{% extends "layout.html" %}
{% block body %}
        {% include "notifications.html" %}
        <div class="col-sm-9 col-sm-offset-3 col-md-10 col-md-offset-2 main">
          {% include "small_topbar.html" %}
          <ul class="breadcrumb" style="margin-bottom: 5px;">
            {% if server -%}
            <li><a href="{{ url_for('view.home') }}">Home</a></li>
            <li class="active">Calendar</li>
            {% else -%}
            <li><a href="{{ url_for('view.home') }}">Home</a></li>
            <li class="active">Calendar</li>
            {% endif -%}
          </ul>
          <br />
          <h1 class="page-header">Calendar{% if cname %} of {{ cname }}{% if server %} on {{ server }}{% endif %}{% elif server %} of {{ server }}{% endif %}</h1>
          <div class="row placeholders" ng-controller="CalendarCtrl">
            <div ui-calendar="uiConfig.calendar" ng-model="eventSources" calendar="backups" id="bui-calendar"></div>
          </div>
        </div>
{% endblock %}
