# -*- coding: utf8 -*-
"""
.. module:: burpui.ext.sql
    :platform: Unix
    :synopsis: Burp-UI external SQL module.

.. moduleauthor:: Ziirish <hi+burpui@ziirish.me>

"""
from flask_sqlalchemy import SQLAlchemy

db = SQLAlchemy()
