{% extends "layout.html" %}
{% block body %}
        <div class="col-sm-12 col-md-12 main" ng-controller="AboutCtrl" ng-cloak>
					<br />
					<h1 class="page-header">About</h1>
					<p>
						<a href="{{ url }}">Burp-UI</a> is
						brought to you by <a href="https://ziirish.info/">Ziirish</a>.
						<br />
						You can find the documentation at <a href="{{ doc }}">{{ doc }}</a>.
					</p>
					{# From here, the jinja syntax is escaped because we use the angularjs syntax #}
					{% raw %}
					<p>
						<strong>Burp-UI</strong> version: <em>{{ version }}</em>
					</p>
					<p ng-repeat="b in burp" ng-if="b.client || b.server">
						<strong>{{ ::b.name }}</strong><span ng-if="b.client"> - client: <em>{{ ::b.client }}</em></span><span ng-if="b.server"> - server: <em>{{ ::b.server }}</em></span>
					</p>
					<p>
					API <a ng-href="{{ api }}">documentation</a>
					</p>
					{% endraw %}
				</div>
{% endblock %}
