{% extends "layout.html" %}
{% block body %}
        {% include "notifications.html" %}
        <div class="col-sm-9 col-sm-offset-3 col-md-10 col-md-offset-2 main">
          {% include "small_topbar.html" %}
          <ul class="breadcrumb" style="margin-bottom: 5px;">
            {% if server -%}
            <li><a href="{{ url_for('view.home') }}">Home</a></li>
            <li class="active">{{ server }} clients</li>
            {% else -%}
            <li class="active">Home</li>
            {% endif -%}
          </ul>
          <br />
          <h1 class="page-header">Global server report</h1>
          <div class="row placeholders">
            <div class="col-xs-6 col-sm-6 placeholder">
              <div class="mycharts" id="chart_repartition" style="height: 350px;">
                <svg></svg>
              </div>
              <h4>Clients repartition</h4>
            </div>
            <div class="col-xs-6 col-sm-6 placeholder">
              <div class="mycharts" id="chart_size" style="height: 350px;">
                <svg></svg>
              </div>
              <h4>Used space</h4>
            </div>
            <div class="col-xs-6 col-sm-6 placeholder">
              <div class="mycharts" id="chart_files" style="height: 350px;">
                <svg></svg>
              </div>
              <h4>Total files</h4>
            </div>
            <div class="col-xs-6 col-sm-6 placeholder">
              <div class="mycharts" id="chart_backups" style="height: 350px;">
                <svg></svg>
              </div>
              <h4>Number of backups</h4>
            </div>
          </div>
        </div>
{% endblock %}
